package com.ruoyi.oa.service;

import java.util.List;
import java.util.Map;

import com.ruoyi.oa.domain.CmcDevice;

/**
 * cmc设备信息Service接口
 * 
 * @author cmc
 * @date 2024-03-05
 */
public interface ICmcDeviceService 
{
    /**
     * 查询cmc设备信息
     * 
     * @param deviceId cmc设备信息主键
     * @return cmc设备信息
     */
    public CmcDevice selectCmcDeviceByDeviceId(Integer deviceId);

    /**
     * 查询cmc设备信息列表
     * 
     * @param cmcDevice cmc设备信息
     * @return cmc设备信息集合
     */
    public List<CmcDevice> selectCmcDeviceList(CmcDevice cmcDevice);

    /**
     * 查询cmc设备名称集合
     *
     * @return cmc设备名称集合
     */
    public List<Map<String,Object>> selectCmcDeviceNameList();

    /**
     * 新增cmc设备信息
     * 
     * @param cmcDevice cmc设备信息
     * @return 结果
     */
    public int insertCmcDevice(CmcDevice cmcDevice);

    /**
     * 修改cmc设备信息
     * 
     * @param cmcDevice cmc设备信息
     * @return 结果
     */
    public int updateCmcDevice(CmcDevice cmcDevice);

    /**
     * 批量删除cmc设备信息
     * 
     * @param deviceIds 需要删除的cmc设备信息主键集合
     * @return 结果
     */
    public int deleteCmcDeviceByDeviceIds(Long[] deviceIds);

    /**
     * 删除cmc设备信息信息
     * 
     * @param deviceId cmc设备信息主键
     * @return 结果
     */
    public int deleteCmcDeviceByDeviceId(Long deviceId);
}
